REM
REM Licensed Materials - Property of IBM
REM Rational ClearQuest
REM (C) Copyright IBM Corp. 2006, 2014. All Rights Reserved.
REM
REM US Government Users Restricted Rights -
REM Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
REM
REM=========================================================================================
REM setup_cq_clob_index.sql - Script to setup clob indexes for Oracle.
REM=========================================================================================
REM
REM Run this file using Oracle SQL interface to setup proper index for CLOB
REM Run this using Oracle user for the ClearQuest database
REM i.e. sqlplus <cquser>/<password>@<tnsname> @setup_cq_clob_index.sql


SET SERVEROUTPUT ON
SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER SQLERROR EXIT
WHENEVER OSERROR EXIT

prompt This utility drops or creates a MultiLineText (CLOB) Index.
prompt Please make sure it is run from a cquser account.
prompt    
prompt Please verify (y) that this is an Oracle 10 or greater database?
ACCEPT database_version CHAR PROMPT 'Enter y or n: '

DECLARE
    ctx_wrong_version EXCEPTION;
    PRAGMA EXCEPTION_INIT(ctx_wrong_version, -20900);
BEGIN
    IF '&database_version' != 'y'
    THEN
        DBMS_OUTPUT.PUT_LINE('****');
        DBMS_OUTPUT.PUT_LINE('**** Oracle 9 is not a supported database for this feature.');
        DBMS_OUTPUT.PUT_LINE('**** Oracle 10 or higher must be the Oracle Version of your database.');
        DBMS_OUTPUT.PUT_LINE('****');
        RAISE_APPLICATION_ERROR(-20900, 'Procedure Terminated.');
    END IF;
END;
/

prompt Do you wish to:
prompt 1   Drop an existing MultiLine Text Index?
prompt 2   Create a new MultiLine Text Index?
ACCEPT create_drop NUMBER FORMAT 9 PROMPT 'Enter 1 or 2: '

REM to self... better way to check for a legal 1 or 2 value and reprompt?  so far can't find one!
BEGIN
    IF &create_drop > 2 or &create_drop < 1
    THEN
        RAISE_APPLICATION_ERROR(-20100, 'Input Invalid for create or drop: &create_drop');
    END IF;
END;
/

REM to self... want to use nvarchar2 variables for globalization, how do I test?

prompt    
accept cqentitydef prompt 'Please enter the CQ EntityDef name: '
accept cqfielddef prompt 'Please enter the CQ FieldDef name for &cqentitydef: '
prompt 
prompt    

REM trim spaces and put into bind variable.
VARIABLE my_bind_cq_entitydef VARCHAR2(30);
EXECUTE :my_bind_cq_entitydef := LOWER(TRIM('&cqentitydef'));
VARIABLE my_bind_cq_fielddef VARCHAR2(30);
EXECUTE :my_bind_cq_fielddef := LOWER(TRIM('&cqfielddef'));

VARIABLE entitydef_db_name varchar2(32);
VARIABLE fielddef_db_name varchar2(32);
VARIABLE lexer_name varchar2(25);
VARIABLE mixed_case varchar2(5);
VARIABLE keep_time varchar2(50);
VARIABLE max25indexname varchar2(25);

REM  Find database names of entitydef and fielddef
BEGIN
    BEGIN
        SELECT distinct e.db_name into :entitydef_db_name from entitydef e where lower(e.name) = :my_bind_cq_entitydef;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            DBMS_OUTPUT.PUT_LINE('****');
            DBMS_OUTPUT.PUT_LINE('**** ERROR: Entitydef: '||:my_bind_cq_entitydef||' not found.');
            DBMS_OUTPUT.PUT_LINE('**** Procedure Terminated. ');
            DBMS_OUTPUT.PUT_LINE('****');
            RAISE_APPLICATION_ERROR(-20200, 'Input Invalid EntityDef specified');
    END;
    BEGIN
        SELECT distinct f.db_name into :fielddef_db_name from entitydef e, fielddef f where lower(e.db_name) = lower(:entitydef_db_name) and lower(f.name) = :my_bind_cq_fielddef and e.id = f.entitydef_id;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            DBMS_OUTPUT.PUT_LINE('****');
            DBMS_OUTPUT.PUT_LINE('**** ERROR: Fielddef: '||:my_bind_cq_fielddef||' not found.');
            DBMS_OUTPUT.PUT_LINE('**** Procedure Terminated. ');
            DBMS_OUTPUT.PUT_LINE('****');
            RAISE_APPLICATION_ERROR(-20210, 'Input Invalid FieldDef specified');
    END;
END;
/

REM Generate a new CLOB index name "CQ_<TABLENAME>_<FIELDNAME>_<COUNTER>".
REM
REM Oracle objects can have 30 character names, but Oracle puts its own
REM 3 character prefix and 2 character suffix on text indexes so only 25
REM characters are available.  To make a new index, 9 characters are used
REM for TABLENAME and INDEXNAME, and 2 characters for COUNTER, then if
REM the generated name is already in use, COUNTER is incremented.

DECLARE
    counter number;
    numfound number;
    table_part varchar(9);
    column_part varchar(9);
BEGIN
    numfound := 0;
    counter := 0;
    table_part := substr(:entitydef_db_name, 1, 9);
    column_part := substr(:fielddef_db_name, 1, 9);
    :max25indexname := 'cq_' || table_part || '_' || column_part || '_' || to_char(counter);
    SELECT COUNT(*) INTO numfound FROM user_indexes WHERE index_name = :max25indexname; 
    WHILE (numfound > 0)
    LOOP
        counter := counter + 1;
        :max25indexname := 'cq_' || table_part || '_' || column_part || '_' || to_char(counter);
        IF (counter >= 100)
        THEN
            RAISE_APPLICATION_ERROR(-20210, 'Too many inde_name conflicts:  max25indexname = ' || :max25indexname);
        END IF;
        SELECT COUNT(*) INTO numfound FROM user_indexes WHERE index_name = :max25indexname; 
    END LOOP;
END;
/

-- prompt Index Drop now being processed...

DECLARE
    index_does_not_exist EXCEPTION;
    PRAGMA EXCEPTION_INIT(index_does_not_exist, -1418);
BEGIN
    IF &create_drop = 1
    THEN
        select to_char(sysdate, 'Dy DD-Mon-YYYY HH24:MI:SS') into :keep_time from dual;
        DBMS_OUTPUT.PUT_LINE('Starting time of index drop is '||:keep_time);
        DBMS_OUTPUT.PUT_LINE('CQ EntityDef table name: ' || :entitydef_db_name);
        DBMS_OUTPUT.PUT_LINE('CQ FieldDef column name: ' || :fielddef_db_name);
        SELECT index_name INTO :max25indexname FROM user_ind_columns WHERE table_name = upper(:entitydef_db_name) AND column_name = upper(:fielddef_db_name);
        DBMS_OUTPUT.PUT_LINE('Removing FieldDef index: ' || :max25indexname);
        EXECUTE immediate 'drop index '||:max25indexname;
        select to_char(sysdate, 'Dy DD-Mon-YYYY HH24:MI:SS') into :keep_time from dual;
        DBMS_OUTPUT.PUT_LINE('Ending time of index drop is '||:keep_time);
    END IF;
EXCEPTION
    WHEN index_does_not_exist THEN
        DBMS_OUTPUT.PUT_LINE('****');
        DBMS_OUTPUT.PUT_LINE('**** ERROR: index: '||:max25indexname||' does_not_exist');
        DBMS_OUTPUT.PUT_LINE('**** Procedure Terminated. ');
        DBMS_OUTPUT.PUT_LINE('****');
END;
/

REM  SQLPlus kludgy way to execute an if.  Note that cq_quit.sql and cq_do_nothing.sql scripts need to be present in same directory as this script.
SET TERMOUT OFF
COLUMN exit_or_not_col NEW_VALUE exit_or_not_script NOPRINT
SELECT DECODE (TRIM('&create_drop'), '1', 'cq_quit.sql', '2', 'cq_do_nothing.sql', 'ERROR') exit_or_not_col FROM dual;
SET TERMOUT ON
@@&exit_or_not_script

REM rest of this script is now for CREATE only...

prompt
prompt Please enter Supported Language Lexer type:
prompt 
prompt 1     BASIC_LEXER        - English and most western European languages that use white space delimited words. 
prompt                            This is the only lexer which supports case controllability.
prompt
prompt 2     CHINESE_LEXER      - Lexer for extracting tokens from Chinese text.
prompt 3     CHINESE_VGRAM_LEXER      - Lexer for extracting tokens from Chinese text.
prompt 4     JAPANESE_LEXER     - Lexer for extracting tokens from Japanese text.
prompt 5     JAPANESE_VGRAM_LEXER     - Lexer for extracting tokens from Japanese text.
prompt 6     KOREAN_MORPH_LEXER - Lexer for extracting tokens from Korean text.
prompt 7     KOREAN_LEXER       - Lexer for extracting tokens from Korean text.
REM prompt ?     MULTI_LEXER        - Lexer for indexing mixed languages such as English, German, and Japanese.
REM prompt ?     USER_LEXER         - Lexer you create to index a particular language.
ACCEPT lexer_type NUMBER FORMAT 9 PROMPT 'Enter 1 through 7: '

BEGIN
    IF &lexer_type > 7 or &lexer_type < 1
    THEN
        RAISE_APPLICATION_ERROR(-20300, 'Input Invalid for lexer type: &lexer_type');
    ELSIF &lexer_type = 1
    THEN
        :lexer_name := 'BASIC_LEXER';
    ELSIF &lexer_type = 2
    THEN
        :lexer_name := 'CHINESE_LEXER';
    ELSIF &lexer_type = 3
    THEN
        :lexer_name := 'CHINESE_VGRAM_LEXER';
    ELSIF &lexer_type = 4
    THEN
        :lexer_name := 'JAPANESE_LEXER';
    ELSIF &lexer_type = 5
    THEN
        :lexer_name := 'JAPANESE_VGRAM_LEXER';
    ELSIF &lexer_type = 6
    THEN
        :lexer_name := 'KOREAN_MORPH_LEXER';
    ELSIF &lexer_type = 7
    THEN
        :lexer_name := 'KOREAN_LEXER';
    ELSE
        RAISE_APPLICATION_ERROR(-20310, 'Input Invalid for lexer type: &lexer_type');
    END IF;
END;
/

-- TODO:  NEED a way to only do this for BASIC_LEXER  ???
REM  SQLPlus kludgy way to execute an if.  Note that cq_prompt_for_case.sql and cq_do_nothing.sql scripts need to be present in same directory as this script.
SET TERMOUT OFF
COLUMN exit_or_not_col NEW_VALUE exit_or_not_script NOPRINT
SELECT DECODE (TRIM('&lexer_type'), '1', 'cq_prompt_for_case.sql', 'cq_do_nothing.sql') exit_or_not_col FROM dual;
SET TERMOUT ON
@@&exit_or_not_script

-- This section is to ensure setup for privileges routine has been run, since ctx_ddl not found does not return an actual error code.
VARIABLE priv_found NUMBER;
DECLARE
    ctx_indexes_no_select EXCEPTION;
    PRAGMA EXCEPTION_INIT(ctx_indexes_no_select, -20400);
BEGIN
    SELECT COUNT(*) into :priv_found FROM user_tab_privs WHERE table_name = 'CTX_INDEXES' AND privilege = 'SELECT';
    --select count(*) into :priv_found from ctxsys.ctx_indexes;
    --select f into :priv_found from foo;
    IF (:priv_found = 0) 
        THEN
        DBMS_OUTPUT.PUT_LINE('****');
        DBMS_OUTPUT.PUT_LINE('**** ERROR: SELECT Privilege can not be found for CTX_INDEXES.');
        DBMS_OUTPUT.PUT_LINE('****        Please run setup_cq_ora_user_clob.sql before running this utility.');
        DBMS_OUTPUT.PUT_LINE('****');
        RAISE_APPLICATION_ERROR(-20400, 'Procedure Terminated.');
    END IF;
EXCEPTION
    WHEN ctx_indexes_no_select 
        THEN
        RAISE;
    WHEN OTHERS
        THEN
        RAISE_APPLICATION_ERROR(-20500, '**** ERROR: Unexpected System Error found.  Please Contact Technical Support.');
END;
/
 
-- drop preference if it exists
--TODO need to test if any ill effects of dropping preference on existing index cause problem???
WHENEVER SQLERROR CONTINUE;
DECLARE
    preference_does_not_exist EXCEPTION;
    PRAGMA EXCEPTION_INIT(preference_does_not_exist, -20000);
BEGIN
    ctx_ddl.drop_preference('cq_lexer_casecontrol');
EXCEPTION
    WHEN preference_does_not_exist THEN
        --      DBMS_OUTPUT.PUT_LINE('debug');
        DBMS_OUTPUT.PUT_LINE('No preference found to drop.  Proceeding.');
END;
/
WHENEVER SQLERROR EXIT

DECLARE
    holdastring varchar(250);
    --cursor_handle INTEGER := DBMS_SQL.OPEN_CURSOR;
BEGIN
    holdastring := 'BEGIN ctx_ddl.create_preference(''cq_lexer_casecontrol'','''||TRIM(:lexer_name)||'''); END;';
    -- dbms_output.put_line('debug, holdstring: '||holdastring);
    execute immediate holdastring;
    -- only do case attribute for BASIC_LEXER
    IF &lexer_type = 1
    THEN
        holdastring := 'BEGIN ctx_ddl.set_attribute(''cq_lexer_casecontrol'', ''mixed_case'','''||TRIM(:mixed_case)||''' ); END;';
        --dbms_output.put_line('debug, holdstring: '||holdastring);
        execute immediate holdastring;
    END IF;
END;
/

prompt
prompt Please enter how you wish the non-alphanumeric characters(,&=?{}\()[]-;~|$!>*%_@`#^</:.) be treated when index is created:
prompt 1   Ignore the non-alphanumeric characters.
prompt 2   Treat the non-alphanumeric characters as normal alphanumeric characters.
ACCEPT non_alphanumeric_select NUMBER FORMAT 9 PROMPT 'Enter 1 or 2: '
DECLARE
    holdastring varchar(250);
    --cursor_handle INTEGER := DBMS_SQL.OPEN_CURSOR;
BEGIN
IF &non_alphanumeric_select > 2 or &non_alphanumeric_select < 1
THEN
	RAISE_APPLICATION_ERROR(-20600, 'Input Invalid for non-alphanumeric select: &case_of_index');
ELSIF &non_alphanumeric_select = 2
THEN
    holdastring := 'BEGIN ctx_ddl.set_attribute(''cq_lexer_casecontrol'', ''printjoins'','',&=?{}\()[]-;~|$!>*%_@`#^</:.'' ); END;';
    --dbms_output.put_line('debug, holdstring: '||holdastring);
    execute immediate holdastring;
END IF;
END;
/


prompt
prompt Index Creation now being processed...
DECLARE 
    holdastring VARCHAR(250);
BEGIN
    dbms_output.put_line('');
    select to_char(sysdate, 'Dy DD-Mon-YYYY HH24:MI:SS') into :keep_time from dual;
    dbms_output.put_line('Starting time of index create is '||:keep_time);
    DBMS_OUTPUT.PUT_LINE('CQ EntityDef table name: ' || :entitydef_db_name);
    DBMS_OUTPUT.PUT_LINE('CQ FieldDef column name: ' || :fielddef_db_name);
    DBMS_OUTPUT.PUT_LINE('Creating FieldDef index: ' || :max25indexname);
    holdastring := 'create index '||:max25indexname||' on '||:entitydef_db_name||'('||:fielddef_db_name||') indextype is ctxsys.context parameters (''lexer cq_lexer_casecontrol sync (on commit)'')';
    DECLARE
        name_already_used EXCEPTION;
        PRAGMA EXCEPTION_INIT(name_already_used, -955);
    BEGIN
        dbms_output.put_line('The index is being created as: '||holdastring);
        execute immediate holdastring; 
        select to_char(sysdate, 'Dy DD-Mon-YYYY HH24:MI:SS') into :keep_time from dual;
        dbms_output.put_line('Ending time of index create is '||:keep_time);
    EXCEPTION
        WHEN name_already_used THEN
            DBMS_OUTPUT.PUT_LINE('****');
            DBMS_OUTPUT.PUT_LINE('**** ERROR: index name '||:max25indexname||' is already used by an existing object');
            DBMS_OUTPUT.PUT_LINE('**** NOTE:  Perhaps an incomplete cleanup of an earlier attempt to create?  Consider dropping this object.');
            DBMS_OUTPUT.PUT_LINE('**** Procedure Terminated. ');
            DBMS_OUTPUT.PUT_LINE('****');
    END;

END; 
/ 

quit;
